//package ij.plugin;

import ij.IJ;
import ij.ImageJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.NewImage;
import ij.plugin.filter.PlugInFilter;
import ij.process.ImageProcessor;

//import jRenderer3D.JRenderer3D;
//import jRenderer3D.Text3D;
//import jRenderer3D.Transform;
import jRenderer3D.*;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.awt.image.MemoryImageSource;
//import java.awt.event.WindowListener;
//import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import ij.gui.GenericDialog;
import ij.gui.ProgressBar;
import javax.swing.border.BevelBorder;
import ij.gui.ImageWindow;
import ij.macro.Interpreter;
import ij.Menus;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;


/**
 *
 *
 */
public class SurfacePlotter_3D implements PlugInFilter, MouseListener, MouseMotionListener{

        // constants
        private final String DOTS = "Dots";
        private final String LINES = "Lines";
        private final String MESH = "Mesh";
        private final String FILLED = "Filled";
        protected String plotType = DOTS;

        private final String ORIGINAL = "Original Colors";
        private final String GRAYSCALE = "Grayscale";
        private final String SPECTRUM = "Spectrum LUT";
        private final String FIRE = "Fire LUT";
        private final String THERMAL = "Thermal LUT";
        private final String ORANGE = "Orange";
        private final String BLUE = "Blue";
        private String colorType = ORIGINAL;

        // application window
        private JFrame frame;


        // panels
        private JPanel mainPanel;
        private JPanel settingsPanel;


        // setting components
        private JComboBox comboDisplayType;
        private JComboBox comboDisplayColors;
        private JCheckBox checkBox;

        private JSlider sliderLight;
        private JSlider sliderGridSize;
        private JSlider sliderSmoothing;
        private JSlider sliderScale, currentSlider;


        // image panel / canvas
        private ImageRegion imageRegion;


        // imageJ components
        private ImagePlus image;


        // imgeJ3D API components
        private JRenderer3D jRenderer3D;


        // other global params
        private int startWidth = 512;
        private int startHeight = 512;
        private double scaleWindow = 1.; // scaling caused by the resize

        //  do not change sizes !
        //private final static int H = 512;
        //private final static int WR = 512;
        //private final static int WL = 200;
        //
        //private String IMAGE = "plot2.jpg";
        private Transform tr;
        private int xStart;
        private int yStart;
        private boolean drag, isCoordinate=false;
        //private boolean addCoordinate=false;
        private int xdiff;
        private int ydiff;
        private final int gridSizeMinValue=3;
        private final int gridSizeMaxValue=9;
        private double light = 0.5;
        private ImageJ ij;
        double x=0, y=0, z=0;
        private BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
        private Border grayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.gray, Color.lightGray);
        private Border lightGrayBorder = BorderFactory.createBevelBorder(bb.LOWERED, Color.lightGray, Color.lightGray);



        public static void main(String args[]) {
                SurfacePlotter_3D sp = new SurfacePlotter_3D();

                //sp.loadImage();
                sp.generateSampleImage();
                sp.runApplication();
        }

        private void generateSampleImage() {

                int width = 320;
                int height = 256;
                int [] pixels = new int [width*height];

                for (int y = 0; y < height; y++) {
                        int dy1 = y - 180;
                        int dy2 = y - 160;

                        for (int x = 0; x < width; x++) {
                                int dx1 = x - 190;
                                int dx2 = x - 70;
                                double r1 = Math.sqrt(dx1*dx1+dy1*dy1)/60;
                                double r2 = Math.sqrt(dx2*dx2+dy2*dy2)/50;

                                int v1 = (int) (250*Math.exp(-r2*r2));
                                int v2 = (int) (250*Math.exp(-r1*r1) );


                                pixels[y*width+x] = 0xFF000000 | ((int)(v2+v1)<<16)| (int)(v2 << 8) | (128 - y/2);
                        }
                }
                MemoryImageSource source = new MemoryImageSource(width, height, pixels, 0, width);

                Image awtImage = Toolkit.getDefaultToolkit().createImage(source);

                BufferedImage bufferedImage =  new BufferedImage(320, 256, BufferedImage.TYPE_INT_RGB);
                Graphics2D g2D = bufferedImage.createGraphics();

                g2D.drawImage(awtImage, 0, 0, null);
                g2D.setColor(Color.BLUE);
                Font font = new Font("Sans", Font.BOLD, 65);
                g2D.setFont(font);
                g2D.drawString("ImageJ2x", 12, 70);

                g2D.dispose();

                image = new ImagePlus("Start_Image", bufferedImage);
        }



        public void run(ImageProcessor imageProcessor) {
                runApplication();
        }

        public int setup(String arg, ImagePlus imp) {
        this.image = imp;

        if (imp == null) {
                return NO_CHANGES;//generateSampleImage();
                }
        return NO_IMAGE_REQUIRED; // | NO_CHANGES;
        }


        private void runApplication() {
                // create window/frame
                frame = new JFrame("Interactive Surface Plot");

                // create application gui
                createGUI();

                // creates the 3d renderer // NOTE: image must be loaded
                create3DRenderer();

                //frame.pack();
        }



        /********************************************************
         * 														*
         *						3D RENDERER						*
         *														*
         ********************************************************/

        /**
         * Initializes the JRenderer3D. Set Background, the surface plot, plot mode, lightning mode.
         * Adds a coordinate system. Sets scale. Renders and updates the image.
         */
        private void create3DRenderer(){

                int iw = image.getWidth();
                int ih = image.getHeight();
                int id = 256;

                int wc = (iw+1)/2;
                int hc = (ih+1)/2;
                int dc = (id+1)/2;

                //tr = new Transform(WR, H);
                // create 3D renderer
                // center in the middle of the image
                jRenderer3D = new JRenderer3D(wc, hc, dc);
                //jRenderer3D.setBackgroundColor(Color.LIGHT_GRAY);

                // surfaceplot
                jRenderer3D.setSurfacePlotGridSize(iw,ih);
                jRenderer3D.setSurfacePlot(image);
                jRenderer3D.setSurfacePlotLight(sliderLight.getValue()/100.);
                jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_DOTSNOLIGHT);

                jRenderer3D.setTransformScale(scaleWindow*sliderScale.getValue()/10.);
                if(!isCoordinate)
                addCoordinateSystem(iw, ih, id);
                renderAndUpdateDisplay();
        }

        /**
         * Draws a simple coordinate system and labels it
         * @param iw
         * @param ih
         * @param id
         */
        private void addCoordinateSystem(int iw, int ih, int id){

//		 add coordinate system
                jRenderer3D.add3DCube(0, 0, 0, iw, ih, id, Color.WHITE);

                //		 add text to the coordinate system
                int d = (int) (0.07*iw); // text position offset

                jRenderer3D.addText3D(new Text3D("0",   -d,   -d,  -d, Color.BLACK , 16));
                jRenderer3D.addText3D(new Text3D("x",    iw+d, -d,  -d, Color.ORANGE, 16));
                jRenderer3D.addText3D(new Text3D("y",   -d,  ih+d,  -d, Color.ORANGE, 16));
                jRenderer3D.addText3D(new Text3D("Lum", -d,   -d, id+d, Color.ORANGE, 16));
        }

        /**
         * Renders and updates the 3D image.
         * Image region is repainted.
         *
         */
        private void renderAndUpdateDisplay() {
                jRenderer3D.doRendering();
                imageRegion.setImage(jRenderer3D);
                imageRegion.repaint();
        }




        /************************************************************
         * 							    *
         *					ACTION MOUSELISTENER*
         *							    *
         ************************************************************/

        public void mouseClicked(MouseEvent e) {
                Object source = e.getSource();
                if (source == imageRegion) { // top view

                        if(e.getClickCount() > 1) {
                                jRenderer3D.setTransformRotationXYZ(0,0,0);
                                renderAndUpdateDisplay();
                        }
                }
        }

        public void mousePressed(MouseEvent e) {
                Object source = e.getSource();

                if (source == imageRegion) {
                        xStart = e.getX();
                        yStart = e.getY();
                        xdiff = 0;
                        ydiff = 0;
                        drag = true;
                        //sliderGridSize.setValue(sliderGridSize.getValue()-1);
                        jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_DOTSNOLIGHT);
                        renderAndUpdateDisplay();
                }

        }

        public void mouseReleased(MouseEvent e) {
                //Object source = arg0.getSource();
                drag = false;
                //sliderGridSize.setValue(sliderGridSize.getValue()+1);
                setPlotType(plotType);
                setColorType(colorType);
                jRenderer3D.setSurfacePlotLight(light);
                renderAndUpdateDisplay();
        }

        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}

        /******************************************************************
         * 			                                          *
         *					ACTION MOUSEMOTIONLISTENER*
         *								  *
         ******************************************************************/

        public void mouseMoved(MouseEvent e) {}

        public void mouseDragged(MouseEvent e) {
                Object source = e.getSource();
                if (source == imageRegion) {
                        if (drag == true) {
                                int xAct = e.getX();
                                int yAct = e.getY();
                                xdiff = xAct - xStart;
                                ydiff = yAct - yStart;
                                xStart = xAct;
                                yStart = yAct;

                                jRenderer3D.changeTransformRotationXZ(-ydiff/2, xdiff/2);

                                renderAndUpdateDisplay();
                        }
                }
        }

        /******************************************************************
         * 								  *
         *					END ACTION LISTENER	  *
         *								  *
         ******************************************************************/


        /**
         * Sets the surface plot mode. Renders and updates the image.
         *
         */
        private void setPlotType(String type) {
                if (type.equals(DOTS))
                        //jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_DOTS);
                        jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_DOTSNOLIGHT);
                else if (type.equals(LINES))
                        jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_LINES);
                else if (type.equals(MESH))
                        jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_MESH);
                else if (type.equals(FILLED))
                        jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_FILLED);

                renderAndUpdateDisplay();
        }

        /**
         * Gets the surface plot mode. Renders and updates the image.
         *
         */
        private String getPlotType() {
          return plotType;
        }

        /**
         * Sets the surface color type.
         * Sets the surface plot mode.
         * Updates illumination, smoothing and scaling and finaly Renders and updates the image.
         *
         */
        private void setChooseAndSliderValue(String ctype, String type, JSlider currentSlider) {
            // Sets the surface color type.
            boolean trueType = true;
            colorType = ctype;
            if (ctype.equals(ORIGINAL))
                    jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_ORIGINAL);
            else if (ctype.equals(GRAYSCALE))
                    jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_GRAY);
            else if (ctype.equals(SPECTRUM))
                    jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_SPECTRUM);
            else if (ctype.equals(FIRE))
                    jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_FIRE);
            else if (ctype.equals(THERMAL))
                    jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_THERMAL);
            else if (ctype.equals(ORANGE))
                    jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_ORANGE);
            else if (ctype.equals(BLUE))
                    jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_BLUE);

                // Sets the surface plot mode.
                    if (type.equals(DOTS))
                            //jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_DOTS);
                            jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_DOTSNOLIGHT);
                    else if (type.equals(LINES))
                            jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_LINES);
                    else if (type.equals(MESH))
                            jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_MESH);
                    else if (type.equals(FILLED))
                            jRenderer3D.setSurfacePlotMode(JRenderer3D.SURFACEPLOT_FILLED);

                        // Updates illumination, smoothing and scaling and finaly Renders and updates the image.
                        for(int i=0; i < 4; i++){
                            if (i == 0) {
                                light = sliderLight.getValue() / 100.;
                                jRenderer3D.setSurfacePlotLight(light);
                            } else if (i == 1) {
                                int grid = 1 << sliderGridSize.getValue();
                                jRenderer3D.setSurfacePlotGridSize(grid, grid);
                                int smooth = (int) ((sliderGridSize.getValue() - 2) * 0.5 *
                                                    sliderSmoothing.getValue());
                                if (grid <= 64) {
                                    jRenderer3D.setSurfaceSmoothingFactor(smooth);
                                }

                                setColorType(colorType);
                                renderAndUpdateDisplay();
                            } else if (i == 2) {
                                int smooth = (int) ((sliderGridSize.getValue() - 2) * 0.5 *
                                                    sliderSmoothing.getValue());
                                if ((1 << sliderGridSize.getValue()) <= 64) {
                                    jRenderer3D.setSurfaceSmoothingFactor(smooth);
                                }
                            } else if (i == 3) {
                                jRenderer3D.setTransformScale(scaleWindow *
                                                              sliderScale.getValue() / 10.);
                            }
                        }
                        renderAndUpdateDisplay();
        }
        /**
         * Sets the surface color type. Renders and updates the image.
         *
         */
        private void setColorType(String type) {
                colorType = type;
                if (type.equals(ORIGINAL))
                        jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_ORIGINAL);
                else if (type.equals(GRAYSCALE))
                        jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_GRAY);
                else if (type.equals(SPECTRUM))
                        jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_SPECTRUM);
                else if (type.equals(FIRE))
                        jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_FIRE);
                else if (type.equals(THERMAL))
                        jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_THERMAL);
                else if (type.equals(ORANGE))
                        jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_ORANGE);
                else if (type.equals(BLUE))
                        jRenderer3D.setSurfacePlotLut(JRenderer3D.LUT_BLUE);

                renderAndUpdateDisplay();
        }




        /**
         * Updates illumination, smoothing and scaling. Renders and updates the image.
         *
         */
        private void sliderChange(JSlider slider) {
                if (slider == sliderLight) {
                        light = sliderLight.getValue() / 100.;
                        jRenderer3D.setSurfacePlotLight(light);
                }
                else if (slider == sliderGridSize) {
                        int grid = 1 << sliderGridSize.getValue();
                        jRenderer3D.setSurfacePlotGridSize(grid, grid);
                        int smooth = (int) ((sliderGridSize.getValue() - 2)*0.5 * sliderSmoothing.getValue());
                        if (!slider.getValueIsAdjusting() || grid <= 64) {
                                jRenderer3D.setSurfaceSmoothingFactor(smooth);
                        }

                        setColorType(colorType);
                        renderAndUpdateDisplay();
                }
                else if (slider == sliderSmoothing) {
                        int smooth = (int) ((sliderGridSize.getValue() - 2)*0.5 * sliderSmoothing.getValue());
                        if (!slider.getValueIsAdjusting() || (1 << sliderGridSize.getValue()) <= 64) {
                                jRenderer3D.setSurfaceSmoothingFactor(smooth);
                        }
                }
                else if (slider == sliderScale){
                    jRenderer3D.setTransformScale(scaleWindow *
                                                  sliderScale.getValue() / 10.);
                }
                renderAndUpdateDisplay();
        }




        /**
         * Rezises the buffer size of the image. Renders and updates image.
         *
         */
        public void resizeImagePanel(int width, int height){
                if (jRenderer3D != null) {
                        scaleWindow = Math.min(width, height)/(double)startHeight;

                        jRenderer3D.setBufferSize(width, height);
                        jRenderer3D.setTransformScale(scaleWindow*sliderScale.getValue()/10.);

                        renderAndUpdateDisplay();
                }
        }





        /********************************************************
         * 														*
         *						GUI-METHODS						*
         *														*
         ********************************************************/

        private void createGUI() {
                ij = IJ.getInstance();
                // add window listener
                frame.addWindowListener(new WindowAdapter() {
                        public void windowClosing(WindowEvent event) {
                                frame.dispose();
                        }
                });

                // add component/resize listener
                frame.addComponentListener(new ComponentAdapter(){
                        public void componentResized(ComponentEvent event) {
                                Insets insetsFrame = frame.getInsets();
                                int width = frame.getWidth() - insetsFrame.left - insetsFrame.right;
                                int height = frame.getHeight() - insetsFrame.bottom - insetsFrame.top - settingsPanel.getHeight();
                                if (height>0 && width > 0)
                                        resizeImagePanel(width, height);
                                frame.pack();
                        }
                });

                if (ij!=null && !IJ.isMacintosh()) {
                    Image img = ij.getIconImage();
                    if (img != null)
                        try {
                            frame.setIconImage(img);
                        } catch (Exception e) {}
                }
                // create window/frame content
                mainPanel = createMainPanel();


                // add content to window/frame
                //frame.add(mainPanel);

                frame.getContentPane().add(mainPanel);
                frame.pack();


                //	set size and visibility of frame
                frame.setSize(startWidth, startHeight);
                frame.setResizable(true);

                // get screen dimensions
                Toolkit toolkit = Toolkit.getDefaultToolkit();
                Dimension screenSize = toolkit.getScreenSize();
                int screenWidth = screenSize.width;
                int screenHeight = screenSize.height;

                // center frame on screen
                int centerX = screenWidth/2-startWidth/2;
                int centerY = screenHeight/2-startHeight/2;
                frame.setLocation(centerX, centerY);
                frame.setVisible(true);
        }


        private JPanel createMainPanel(){
                // create image region panel
                JPanel imagePanel = createImagePanel();

                // create settings panel
                JPanel settingsPanel = createSettingsPanel();

                // create slider panel
                Dimension sliderDim = new Dimension(50, 512);
                JPanel sliderPanel = createSliderPanel();
                sliderPanel.setPreferredSize(sliderDim);
                sliderPanel.setBorder(lightGrayBorder);

                // create main panel
                JPanel mainPanel = new JPanel();
                mainPanel.setLayout(new BorderLayout());

                // add panels to main panel
                //GridLayout gl = new GridLayout(1, 2, 0, 0);
                JPanel subPanel = new JPanel();
                subPanel.setLayout(new FlowLayout());
                subPanel.add(imagePanel);
                subPanel.add(sliderPanel);
                mainPanel.add(settingsPanel, BorderLayout.NORTH);
                mainPanel.add(subPanel, BorderLayout.CENTER);

                return mainPanel;
        }


        private JPanel createImagePanel(){
                // create image region
                imageRegion = new ImageRegion();

                // init size
                imageRegion.setWidth(512);
                imageRegion.setHeight(512);

                imageRegion.addMouseListener(this);
                imageRegion.addMouseMotionListener(this);
                imageRegion.addMouseMotionListener(this);

                // create image region panel/canvas
                JPanel panel = new JPanel();
                panel.setLayout(new BorderLayout());
                panel.setBorder(grayBorder);

                // add image region to panel
                panel.add(imageRegion, BorderLayout.CENTER);

                // return image panel
                return panel;
        }


        private JPanel createSettingsPanel(){
                Dimension headDim = new Dimension(512, 25);
                Dimension comboDim = new Dimension(512, 25);
                //Dimension sliderDim = new Dimension(512, 50);

                // create head panel
                JPanel headPanel = new JPanel();
                JLabel headLabel = new JLabel();
                headLabel.setBorder(grayBorder);
                headLabel.setText(" Interactive ControlPanel ");
                headPanel.setLayout(new BorderLayout());
                headPanel.add(BorderLayout.WEST, headLabel);
                headPanel.setPreferredSize(headDim);
                headPanel.setBorder(lightGrayBorder);

                // create combo panel
                JPanel comboPanel = createComboPanel();
                comboPanel.setPreferredSize(comboDim);
                comboPanel.setBorder(lightGrayBorder);

                // create slider panel
                //JPanel sliderPanel = createSliderPanel();
                //sliderPanel.setPreferredSize(sliderDim);
                //sliderPanel.setBorder(lightGrayBorder);

                // create settings panel
                settingsPanel = new JPanel();
                settingsPanel.setBorder(grayBorder);
                settingsPanel.setLayout(new BorderLayout());

                // add combo/slider panels
                settingsPanel.add(headPanel, BorderLayout.NORTH);
                settingsPanel.add(comboPanel, BorderLayout.CENTER);
                //settingsPanel.add(sliderPanel, BorderLayout.SOUTH);

                // return settings panel
                return settingsPanel;
        }


        void createComboDisplay() {
//		 create display colors combo box
                comboDisplayColors = new JComboBox();

                comboDisplayColors.addItem(ORIGINAL);
                comboDisplayColors.addItem(GRAYSCALE);
                comboDisplayColors.addItem(SPECTRUM);
                comboDisplayColors.addItem(FIRE);
                comboDisplayColors.addItem(THERMAL);
                comboDisplayColors.addItem(BLUE);
                comboDisplayColors.addItem(ORANGE);
                comboDisplayColors.setToolTipText("Surface Color Type");

                comboDisplayColors.addActionListener(new ActionListener(){
                        public void actionPerformed(ActionEvent evt) {
                                setColorType(comboDisplayColors.getSelectedItem().toString());
                        }
                });
        }

        private JPanel createComboPanel(){

                // create display type combo box
                comboDisplayType = new JComboBox();

                comboDisplayType.addItem(DOTS);
                comboDisplayType.addItem(LINES);
                comboDisplayType.addItem(MESH);
                comboDisplayType.addItem(FILLED);
                comboDisplayType.setSelectedIndex(0);
                comboDisplayType.setToolTipText("Surface Plot Mode");

                comboDisplayType.addActionListener(new ActionListener(){
                        public void actionPerformed(ActionEvent evt) {
                                plotType = comboDisplayType.getSelectedItem().toString();
                                setPlotType(plotType);
                        }
                });

                createComboDisplay();

                // create save button
                JButton saveButton = new JButton("Duplicate Surface");
                saveButton.setToolTipText("Save Surface to ImageWindow");
                saveButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                imageRegion.saveToImageJImage();
                        }
                });

                // create save button
                JButton resetButton = new JButton("Load Texture");
                resetButton.setToolTipText("Load Texture");
                resetButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                loadTextureImage();
                       }
                });

                checkBox = new JCheckBox("Hide Coord...");
                checkBox.setSelected(!isCoordinate);
                checkBox.setToolTipText("Hide Coordante and Reset");
                checkBox.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
               }
                });
                checkBox.addItemListener(new ItemListener(){
                        public void itemStateChanged(ItemEvent e){
                            if (e.getSource() == checkBox) {
                                isCoordinate=!isCoordinate;
                                //create3DRenderer();
                                //jRenderer3D.setTransformRotationXYZ(x,y,z);
                                //setChooseAndSliderValue(colorType, plotType, currentSlider);
                                    if (checkBox.isSelected () ) {
                                            jRenderer3D.setAxes(true);
                                            jRenderer3D.setText(true);
                                            }
                                    if (!checkBox.isSelected () ) {
                                            jRenderer3D.setAxes(false);
                                            jRenderer3D.setText(false);
                                    }
                            }
                           //setChooseAndSliderValue(colorType, plotType, currentSlider);
                           renderAndUpdateDisplay();
                        }
                });
                // create combo panel
                JPanel panel = new JPanel();
                panel.setLayout(new GridLayout(1, 5, 5, 0));

                // add elements to combo panel
                panel.add(comboDisplayType);
                panel.add(comboDisplayColors);
                panel.add(resetButton);
                panel.add(saveButton);
                panel.add(checkBox);

                // return combo panel
                return panel;
        }


        private void loadTextureImage() {
                JFileChooser fc = new JFileChooser(".");		// open texture image
                if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
                        String str = fc.getSelectedFile().getPath();
                        try {
                                IJ.run("Open...", "path='"+str+"'");
                            ImagePlus impTexture = WindowManager.getCurrentImage();

                                jRenderer3D.setSurfacePlotWithTexture(image, impTexture);

                                comboDisplayColors.setSelectedIndex(0);

                                colorType = ORIGINAL;
                                setPlotType(plotType);
                                setColorType(colorType);
                                jRenderer3D.setSurfacePlotLight(light);

                                renderAndUpdateDisplay();

                        } catch (RuntimeException e) {
                                JOptionPane.showMessageDialog(null,"Error opening Image","",JOptionPane.PLAIN_MESSAGE);
                                return;
                        }
                }
        }


        private JPanel createSliderPanel() {
                // create sliders
                sliderLight = createSlider("Light..", 0, 100, 50, 50, 25, true, true);
                sliderGridSize = createSlider("Grid S..", 3, 9, 9, 3, 1, true, true);
                sliderSmoothing = createSlider("Smoot..", 0, 5, 0, 3, 1, true, true);
                sliderScale = createSlider("Scale", 5, 40, 10, 10, 0, true, true);

                sliderLight.setToolTipText("Lighting");
                sliderGridSize.setToolTipText("Grid Size");
                sliderSmoothing.setToolTipText("Smoothing");
                sliderScale.setToolTipText("Scale");

                // create slider panel
                JPanel panel = new JPanel();
                GridLayout gl = new GridLayout(4, 1, 0, 0);
                panel.setLayout(gl);

                // add elements to combo panel
                panel.add(sliderLight);
                panel.add(sliderGridSize);
                panel.add(sliderSmoothing);
                panel.add(sliderScale);

                // return slider panel
                return panel;
        }


        private JSlider createSlider(String borderTitle, int min, int max, int value, int minor, int major, boolean paintTicks, boolean paintLabels) {
                // create nested border
                Border empty = BorderFactory.createTitledBorder(
                                           BorderFactory.createEmptyBorder());

                // create font for sliders
                Font sliderFont = new Font("Sans", Font.PLAIN, 12);

                // create slider
                JSlider slider = new JSlider(JSlider.VERTICAL, min, max, value);
                /*slider.setBorder(new TitledBorder(
                                empty, borderTitle, TitledBorder.CENTER,
                                TitledBorder.BELOW_TOP,	sliderFont));*/

                slider.setMajorTickSpacing(major);
                slider.setMinorTickSpacing(minor);

                slider.setPaintTicks(paintTicks);
                slider.setPaintLabels(paintLabels);

               slider.setBorder(new TitledBorder(grayBorder, borderTitle));
                slider.addChangeListener(new ChangeListener(){
                        public void stateChanged(ChangeEvent event) {
                            currentSlider=(JSlider)event.getSource();
                                sliderChange(currentSlider);
                        }
                });

                return slider;
        }


        /**
         * Image Region
         */
        class ImageRegion  extends JPanel {

                private Image image;
                private int width;
                private int height;
                private BufferedImage biSrc, destSrc;
                private int plotNumber=1;

                public ImageRegion(){
                }
                public void setImage(JRenderer3D pic){
                        height = pic.getHeight();
                        setWidth(width);
                        setHeight(height);
                        width = pic.getWidth();
                        //System.out.println("width="+width);
                        //System.out.println("height="+height);
                        image = pic.getImage();
                        //createBufferedImage(image);
                        destSrc=biSrc;
                }

                public void setImage(Image image){
                        this.image = image;
                }

                synchronized void saveToImageJImage() {
                        biSrc =  new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
                        paint(biSrc.createGraphics());
                        ImageJ ij = IJ.getInstance();
                        ImageWindow win = WindowManager.getCurrentWindow();
                        if(win == null){plotNumber=0;return;}
                        String imgName = win.getTitle();
                        String endwith = imgName.substring(imgName.indexOf("."));
                        String nlesswe=imgName.substring(0, imgName.length()-imgName.indexOf("."));
                        nlesswe = nlesswe+"Surface-"+plotNumber + endwith;

                        ImagePlus plotImage = NewImage.createRGBImage (nlesswe , width, height, 1, NewImage.FILL_BLACK);

                        ImageProcessor ip = plotImage.getProcessor();

                        int[] pixels = (int[]) ip.getPixels();
                        biSrc.getRGB(0, 0, width, height, pixels, 0, width);

                        plotImage.show();
                        plotImage.updateAndDraw();
                        plotNumber++;
                }

                // This method creates the source and destination images.
                public void createBufferedImage(Image i) {

                    int width = i.getWidth(this);
                    int height = i.getHeight(this);

                    biSrc = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

                    // Draw the texture image into the memory buffer.
                    Graphics2D big = biSrc.createGraphics();
                               big.drawImage(i, 0, 0, width, height, (ImageObserver) this);
                               big.dispose();
                }

                //-------------------------------------------------------------------

                public void paint(Graphics g) {

                       if (image != null ) {
                               g.drawImage(image, 0, 0, width, height, (ImageObserver) this);
                       }
               }

               public void update(Graphics g) {
                        paint(g);
                }
                public Dimension getPreferredSize() {
                        return new Dimension(getWidth(), getHeight());
                }
                public Dimension getMinimumSize() {
                        return new Dimension(getWidth(), getHeight());
                }

                public int getHeight() {
                        return height;
                }
                public void setHeight(int height) {
                        this.height = height;
                }
                public int getWidth() {
                        return width;
                }
                public void setWidth(int width) {
                        this.width = width;
                }
                public BufferedImage getBufferedImage() {
                        return destSrc;
                }

           }

}
